#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
import json
import os
import uuid
from typing import Any, Callable, Dict, List
from enum import Enum
from pydantic import BaseModel
from fastapi import status
from openjiuwen.core.common.logging import logger
from packaging import version
from pydantic import ValidationError

from openjiuwen_studio.core.database import milliseconds
import openjiuwen_studio.core.manager.convertor.plugin as convert
from openjiuwen_studio.core.manager.convertor.components.plugin import param_type_mapping
from openjiuwen_studio.core.manager.login_manager.space import check_user_space
from openjiuwen_studio.core.manager.repositories.plugin_repository import plugin_repository
from openjiuwen_studio.core.manager.repositories.tool_repository import tool_repository
from openjiuwen_studio.core.manager.repositories.agent_repository import agent_repository
from openjiuwen_studio.core.utils.exception import log_exception
from openjiuwen_studio.models.plugin import PluginBaseDBPd, ToolBaseDB, PluginPublishDBPd
from openjiuwen_studio.schemas.plugin import (
    PluginCreate, PluginId, PluginInfo, PluginInfoResponse, PluginApiBase,
    PluginApiInfo, PluginApiInfoCreate, PluginToolId, PluginApiInfoResponse,
    PluginListTool, PluginList, PluginListResponse, PluginListPagination,
    PluginType, PluginToolParam, ToolId, PluginCodeBase, PluginCodeInfo,
    PluginCodeInfoResponse, PluginApiInfoDB, PluginCodeInfoDB, PluginPublish,
    PluginPublishResponse, PluginPublishInfo, PluginPublishListResponse,
    PluginPublishInfoResponse, ParamType
)
from openjiuwen_studio.schemas.common import ResponseModel
from openjiuwen_studio.core.manager.reference_extractor import check_referenced_dependencies
from openjiuwen_studio.core.manager.repositories.reference_repository import reference_repository


def with_exception_handling(func: Callable) -> Callable:
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except ValidationError as e:
            log_exception(e)
            return ResponseModel(
                code=status.HTTP_400_BAD_REQUEST,
                message=type(e).__name__
            )
        except Exception as e:
            log_exception(e)
            return ResponseModel(
                code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                message=type(e).__name__
            )

    return wrapper


@with_exception_handling
def plugin_create(
        req: PluginCreate,
        current_user: dict
) -> ResponseModel:
    """创建新的插件"""
    _ = check_user_space(req.space_id, current_user)

    current_time = milliseconds()

    plugin_id = str(uuid.uuid4())

    plugin_dict = {
        "plugin_id": plugin_id,
        "name": req.name,
        "desc": req.desc,
        "desc_mk": req.desc_mk if hasattr(req, 'desc_mk') else "",
        "url": req.url,
        "icon_uri": req.icon_uri,
        "space_id": req.space_id,
        "plugin_type": req.plugin_type,
        "create_time": current_time,
        "update_time": current_time,
    }

    # 将 request_params 映射到数据库字段 inputs
    if hasattr(req, 'request_params') and req.request_params:
        plugin_dict["inputs"] = [param.model_dump() for param in req.request_params]

    plugin = PluginBaseDBPd(**plugin_dict)
    logger.info(f"create plugin info: {plugin}")

    res = plugin_repository.plugin_create(plugin.model_dump())
    create_result = ResponseModel(**res)
    logger.info(f"create plugin info into db result: {create_result}")
    if create_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=create_result.code,
            message=create_result.message,
        )

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="create plugin success",
        data=PluginId(
            plugin_id=plugin_id,
            space_id=req.space_id,
        )
    )


@with_exception_handling
def plugin_get(
        req: PluginId,
        current_user: dict
) -> ResponseModel:
    """获取插件信息"""
    _ = check_user_space(req.space_id, current_user)

    res, _ = plugin_repository.plugin_get(req.model_dump())
    canvas_result = ResponseModel(**res)
    logger.info(f"get plugin info from db result: {canvas_result}")
    if canvas_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=canvas_result.code,
            message=canvas_result.message,
        )
    # 使用字段映射方法将数据库的 inputs 映射为 request_params
    plugin_info = PluginInfo.from_db_with_mapping(canvas_result.data)
    return ResponseModel(
        code=status.HTTP_200_OK,
        message="get plugin success",
        data=PluginInfoResponse(plugin_info=plugin_info)
    )


@with_exception_handling
def plugin_update(
        req: PluginInfo,
        current_user: dict
) -> ResponseModel:
    """获取插件信息"""
    _ = check_user_space(req.space_id, current_user)

    logger.info(f"update plugin: {req}")
    res, _ = plugin_repository.plugin_get(req.model_dump())
    get_result = ResponseModel(**res)
    logger.info(f"get plugin info from db result: {get_result}")
    if get_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=get_result.code,
            message=get_result.message,
        )
    # Handle both dict and Pydantic model types
    data = get_result.data
    if hasattr(data, 'model_dump'):
        data_dict = data.model_dump()
    elif hasattr(data, 'dict'):
        data_dict = data.dict()
    else:
        data_dict = data
    plugin = PluginBaseDBPd(**data_dict)
    update_dict = {
        "name": req.name,
        "desc": req.desc,
        "url": req.url,
        "icon_uri": req.icon_uri,
    }

    if hasattr(req, 'desc_mk') and req.desc_mk is not None:
        update_dict["desc_mk"] = req.desc_mk

    # 将 request_params 映射到数据库字段 inputs
    if hasattr(req, 'request_params') and req.request_params is not None:
        update_dict["inputs"] = [param.model_dump() for param in req.request_params]

    for key, value in update_dict.items():
        setattr(plugin, key, value)

    res = plugin_repository.plugin_save(plugin.model_dump())
    result = ResponseModel(**res)
    logger.info(f"update plugin info in db result: {result}")
    if result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=result.code,
            message=result.message,
        )

    # 同步更新引用了该插件的Agent中的插件名称
    try:
        if hasattr(req, 'name') and req.name:
            agent_repository.update_plugin_name_in_agents(
                space_id=req.space_id,
                plugin_id=req.plugin_id,
                new_plugin_name=req.name
            )
    except Exception as e:
        logger.error(f"Failed to sync plugin name to agents: {e}", exc_info=True)

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="update plugin success",
    )


@with_exception_handling
def plugin_delete(
        req: PluginId,
        current_user: dict
) -> ResponseModel:
    """删除插件"""
    _ = check_user_space(req.space_id, current_user)

    logger.info(f"delete plugin: {req}")

    # 1. 检查该plugin是否被引用
    try:
        can_delete, message = check_referenced_dependencies(
            reference_repository, req.space_id, "PLUGIN", req.plugin_id
        )

        if not can_delete:
            logger.warning(f"plugin deletion blocked due to dependencies: {req.plugin_id} - {message}")
            return ResponseModel(
                code=status.HTTP_400_BAD_REQUEST,
                message=message,
            )
    except Exception as e:
        logger.error(f"Plugin dependency check failed for {req.plugin_id}: {e}")
        # 依赖检查失败时，为安全起见阻止删除
        return ResponseModel(
            code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            message="Unable to verify plugin dependencies, deletion blocked for safety",
        )

    # 2. 删除所有与该插件相关的工具
    try:
        tools_list_res = tool_repository.tool_list({
            "space_id": req.space_id
        })
        tools_result = ResponseModel(**tools_list_res)

        if tools_result.code == status.HTTP_200_OK and tools_result.data:
            tools = tools_result.data if isinstance(tools_result.data, list) else []

            for tool in tools:
                tool_dict = tool if isinstance(tool, dict) else tool.model_dump() if hasattr(tool,
                                                                                                 'model_dump') else {}
                if tool_dict.get("plugin_id") == req.plugin_id:
                    tool_id = tool_dict.get("tool_id", "")
                    if tool_id:
                        # Use plugin_delete_tool for proper validation and reference checking
                        delete_tool_result = plugin_delete_tool(
                            PluginToolId(
                                space_id=req.space_id,
                                plugin_id=req.plugin_id,
                                tool_id=tool_id
                            ),
                            current_user
                        )
                        if delete_tool_result.code != status.HTTP_200_OK:
                            logger.warning(
                                f"Failed to delete tool {tool_id} for plugin {req.plugin_id}: "
                                f"{delete_tool_result.message}")
    except Exception as e:
        logger.warning(f"Error deleting tools for plugin {req.plugin_id}: {e}", exc_info=True)

    # 3. 执行删除操作
    res = plugin_repository.plugin_delete(req.model_dump())
    delete_result = ResponseModel(**res)
    logger.info(f"delete plugin info in db result: {delete_result}")
    if delete_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=delete_result.code,
            message=delete_result.message,
        )

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="delete plugin success",
    )


@with_exception_handling
def plugin_list(
        req: PluginList,
        current_user: dict
) -> ResponseModel:
    """获取插件列表"""
    _ = check_user_space(req.space_id, current_user)

    res = plugin_repository.plugin_list(req.model_dump())
    list_result = ResponseModel(**res)
    logger.info(f"get plugin list from db result: {list_result}")
    if list_result.code != status.HTTP_200_OK and list_result.code != status.HTTP_404_NOT_FOUND:
        return ResponseModel(
            code=list_result.code,
            message=list_result.message,
        )

    # 处理返回数据
    if list_result.data is None:
        return ResponseModel(
            code=status.HTTP_200_OK,
            message="get plugin success",
            data=PluginListResponse(
                plugin_infos=[],
                pagination={
                    "total": 0,
                    "total_pages": 1,
                    "page": req.page or 1,
                    "page_size": req.size or 10
                }
            )
        )

    # 转换插件信息
    infos: List[PluginInfo] = []
    plugin_data = list_result.data.get("plugin_infos", [])
    for info_dict in plugin_data:
        info = PluginInfo(**info_dict)
        infos.append(info)

    # 获取分页信息
    pagination_data = list_result.data.get("pagination", {})
    pagination = PluginListPagination(**pagination_data) if pagination_data else PluginListPagination(
        total=0,
        total_pages=1,
        page=req.page or 1,
        page_size=req.size or 10
    )

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="get plugin success",
        data=PluginListResponse(
            plugin_infos=infos,
            pagination=pagination
        )
    )


@with_exception_handling
def plugin_convert(
        req: ToolId,
        current_user: dict
) -> ResponseModel:
    """转换插件"""
    _ = check_user_space(req.space_id, current_user)

    get_res, plugin_dict = tool_repository.tool_get(req.model_dump())
    get_result = ResponseModel(**get_res)
    if get_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=get_result.code,
            message=get_result.message,
        )
    tool_info = get_result.data
    plugin = convert.plugin_convert(PluginBaseDBPd(**plugin_dict), tool_info.model_dump())
    return ResponseModel(
        code=status.HTTP_200_OK,
        message="convert plugin success",
        data=plugin
    )


@with_exception_handling
def plugin_create_api(
        req: PluginApiInfoCreate,
        current_user: dict
) -> ResponseModel:
    """创建插件API"""
    _ = check_user_space(req.space_id, current_user)

    logger.info(f"create plugin api info: {req}")

    api_info = PluginApiInfo(
        space_id=req.space_id,
        plugin_id=req.plugin_id,
        plugin_type=PluginType.PLUGIN_TYPE_CLOUD_API,
        tool_id=str(uuid.uuid4()),
        name=req.name,
        desc=req.desc,
        path=req.path,
        method=req.method,
        available=False,
        request_params=req.request_params if hasattr(req, 'request_params') else [],
        response_params=req.response_params if hasattr(req, 'response_params') else [],
        headers=req.headers if hasattr(req, 'headers') else [],
    )

    res = tool_repository.tool_create(api_info.model_dump())
    result = ResponseModel(**res)
    logger.info(f"create plugin api info in db result: {result}")
    if result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=result.code,
            message=result.message,
        )

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="create plugin api success",
        data={"tool_id": api_info.tool_id},
    )


def _plugin_input_output_parameters(params: List[PluginToolParam]) -> List[Dict[str, Any]]:
    input_output_params: List[Dict[str, Any]] = []
    for param in params:
        param_dict = param.model_dump()
        input_output_params.append(param_dict)

    return input_output_params


def _input_parameters_to_request_params(input_parameters: List[Dict[str, Any]]) -> List[PluginToolParam]:
    """Convert input_parameters from database to request_params for API response"""
    if not input_parameters:
        return []

    request_params = []
    for param in input_parameters:
        if isinstance(param.get("type"), str):
            for key, value in param_type_mapping.items():
                if value == param.get('type'):
                    param["type"] = key
                    break
        request_param = PluginToolParam(**param)
        request_params.append(request_param)

    return request_params


def _plugin_update_tool(
        plugin_id: str,
        req: Dict[str, Any]
) -> ResponseModel:
    get_res, _ = tool_repository.tool_get(req)
    get_result = ResponseModel(**get_res)
    if get_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=get_result.code,
            message=get_result.message,
        )

    tool = get_result.data
    # tool is already a dict from database, not a Pydantic model
    tool_plugin_id = tool.get('plugin_id') if isinstance(tool, dict) else tool.plugin_id
    tool_version = tool.get('plugin_version') if isinstance(tool, dict) else tool.plugin_version

    if tool_plugin_id != plugin_id:
        return ResponseModel(
            code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            message="plugin id is not match",
        )

    # 确保 req 中包含 plugin_version，因为唯一约束是 (tool_id, plugin_version)
    # 从已获取的 tool 对象中获取 plugin_version，确保更新时能正确找到记录
    if 'plugin_version' not in req or not req.get('plugin_version'):
        req['plugin_version'] = tool_version if tool_version else ToolBaseDB.__version_none__

    # 确保req中如果没有修改，则不修改available的状态
    # 将 tool 转换为 dict
    if not isinstance(tool, dict):
        tool_dict = tool.to_dict() if hasattr(tool, 'to_dict') else tool.model_dump()
    else:
        tool_dict = tool.copy()

    # 需要对比的字段
    need_compare_fields = {'name', 'desc', 'input_parameters', 'output_parameters', 'plugin_id', 'plugin_type',
                           'plugin_version', 'space_id', 'tool_id', 'headers', 'method', 'path', 'request_params',
                           'response_params'}

    def to_plain(val: Any) -> Any:
        # IntEnum/Enum -> int/基础值
        if isinstance(val, Enum):
            return val.value
        # Pydantic Model -> dict
        if isinstance(val, BaseModel):
            return to_plain(val.model_dump())
        if isinstance(val, dict):
            return {k: to_plain(v) for k, v in val.items()}
        if isinstance(val, list):
            return [to_plain(v) for v in val]
        return val

    # 创建用于比较的字典（指定字段）
    def normalize_for_db_comparison(data: Dict[str, Any]) -> Dict[str, Any]:
        normalized = {}
        for key in need_compare_fields:
            if key in data:
                normalized[key] = to_plain(data[key])
        return normalized

    tool_normalized = normalize_for_db_comparison(tool_dict)
    req_normalized = normalize_for_db_comparison(req)

    # 比较 req 和 tool 的内容
    if tool_normalized != req_normalized:
        # 内容不一致，设置 available=False
        req['available'] = False
    else:
        # 内容一致，保持 available 不变（从数据库中获取的值）
        req['available'] = tool_dict.get('available', False)

    res = tool_repository.tool_save(req)
    result = ResponseModel(**res)
    logger.info(f"update plugin tool info in db result: {result}")
    if result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=result.code,
            message=result.message,
        )

    # 同步更新引用了该工具的Agent中的工具名称
    try:
        if 'name' in req:
            tool_id = req.get('tool_id')
            space_id = req.get('space_id')
            new_name = req.get('name')
            
            if tool_id and space_id and new_name:
                agent_repository.update_tool_name_in_agents(
                    space_id=space_id,
                    tool_id=tool_id,
                    new_tool_name=new_name
                )
    except Exception as e:
        logger.error(f"Failed to sync tool name to agents: {e}", exc_info=True)

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="update plugin tool success",
    )


@with_exception_handling
def plugin_update_api(
        req: PluginApiInfo,
        current_user: dict
) -> ResponseModel:
    """更新插件API"""
    _ = check_user_space(req.space_id, current_user)

    logger.info("update plugin api info")
    update_api = PluginApiInfoDB(**(req.model_dump()))
    update_api.input_parameters = _plugin_input_output_parameters(req.request_params)
    update_api.output_parameters = _plugin_input_output_parameters(req.response_params)
    return _plugin_update_tool(req.plugin_id, update_api.model_dump())


@with_exception_handling
def plugin_delete_tool(
        req: PluginToolId,
        current_user: dict
) -> ResponseModel:
    """删除插件工具"""
    _ = check_user_space(req.space_id, current_user)

    logger.info("delete plugin tool")
    get_res, _ = tool_repository.tool_get(req.model_dump())
    get_result = ResponseModel(**get_res)
    if get_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=get_result.code,
            message=get_result.message,
        )

    tool = get_result.data
    if tool.plugin_id != req.plugin_id:
        return ResponseModel(
            code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            message="plugin id is not match",
        )

    # 检查tool是否被其他资源引用
    can_delete, message = check_referenced_dependencies(
        reference_repository, req.space_id, "TOOL", req.tool_id
    )
    if not can_delete:
        return ResponseModel(
            code=status.HTTP_400_BAD_REQUEST,
            message=message,
        )

    res = tool_repository.tool_delete(req.model_dump())
    result = ResponseModel(**res)
    logger.info(f"delete plugin tool info in db result: {result}")
    if result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=result.code,
            message=result.message,
        )

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="delete plugin tool success",
    )


@with_exception_handling
def plugin_get_api(
        req: PluginToolId,
        current_user: dict
) -> ResponseModel:
    """获取插件API"""
    _ = check_user_space(req.space_id, current_user)

    logger.info("get plugin api")
    get_res, _ = tool_repository.tool_get(req.model_dump())
    get_result = ResponseModel(**get_res)
    if get_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=get_result.code,
            message=get_result.message,
        )

    tool = get_result.data
    if tool.plugin_id != req.plugin_id:
        return ResponseModel(
            code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            message="plugin id is not match",
        )

    # tool is already a dict from database, not a Pydantic model
    api_dict = tool if isinstance(tool, dict) else tool.model_dump()
    if 'available' not in api_dict or api_dict.get('available') is None:
        api_dict['available'] = True

    # Convert input_parameters to request_params with runtime and value
    if 'input_parameters' in api_dict and api_dict['input_parameters']:
        request_params = _input_parameters_to_request_params(api_dict['input_parameters'])
        api_dict['request_params'] = [param.model_dump() for param in request_params]

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="get plugin api success",
        data=PluginApiInfoResponse(
            api_info=[PluginApiInfo(**api_dict)],
            total=1,
        )
    )


@with_exception_handling
def plugin_list_api(
        req: PluginListTool,
        current_user: dict
) -> ResponseModel:
    """获取插件API列表"""
    _ = check_user_space(req.space_id, current_user)

    logger.info("list plugin api")
    list_res, tool_list = plugin_repository.plugin_get(req.model_dump())
    list_result = ResponseModel(**list_res)
    if list_result.code != status.HTTP_200_OK and list_result.code != status.HTTP_404_NOT_FOUND:
        return ResponseModel(
            code=list_result.code,
            message=list_result.message,
        )

    api_infos: List[PluginApiInfo] = []
    for info_dict in tool_list:
        logger.info(f"tool: {info_dict}")
        if 'available' not in info_dict or info_dict.get('available') is None:
            info_dict['available'] = True

        # Convert input_parameters to request_params with runtime and value
        if 'input_parameters' in info_dict and info_dict['input_parameters']:
            request_params = _input_parameters_to_request_params(info_dict['input_parameters'])
            info_dict['request_params'] = [param.model_dump() for param in request_params]

        info = PluginApiInfo(**info_dict)
        if info.plugin_id == req.plugin_id:
            api_infos.append(info)

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="list plugin api success",
        data=PluginApiInfoResponse(
            api_info=api_infos,
            total=len(api_infos),
        )
    )


@with_exception_handling
def plugin_create_code(
        req: PluginCodeBase,
        current_user: dict
) -> ResponseModel:
    """创建插件code工具"""
    _ = check_user_space(req.space_id, current_user)

    logger.info("create plugin code info")

    code_info = PluginCodeInfo(
        space_id=req.space_id,
        plugin_id=req.plugin_id,
        plugin_type=PluginType.PLUGIN_TYPE_CLOUD_CODE,
        tool_id=str(uuid.uuid4()),
        name=req.name,
        desc=req.desc,
        language=req.language,
        code=req.code,
        available=False,
    )

    res = tool_repository.tool_create(code_info.model_dump())
    result = ResponseModel(**res)
    logger.info(f"create plugin code info in db result: {result}")
    if result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=result.code,
            message=result.message,
        )

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="create plugin code success",
        data={"tool_id": code_info.tool_id},
    )


@with_exception_handling
def plugin_update_code(
        req: PluginCodeInfo,
        current_user: dict
) -> ResponseModel:
    """更新插件code工具"""
    _ = check_user_space(req.space_id, current_user)

    logger.info("update plugin code info")
    update_code = PluginCodeInfoDB(**(req.model_dump()))
    update_code.input_parameters = _plugin_input_output_parameters(req.request_params)
    update_code.output_parameters = _plugin_input_output_parameters(req.response_params)
    return _plugin_update_tool(req.plugin_id, update_code.model_dump())


@with_exception_handling
def plugin_get_code(
        req: PluginToolId,
        current_user: dict
) -> ResponseModel:
    """获取插件code工具信息"""
    _ = check_user_space(req.space_id, current_user)

    logger.info("get plugin code")
    get_res, _ = tool_repository.tool_get(req.model_dump())
    get_result = ResponseModel(**get_res)
    if get_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=get_result.code,
            message=get_result.message,
        )

    tool = get_result.data
    # tool is already a dict from database, not a Pydantic model
    tool_plugin_id = tool.get('plugin_id') if isinstance(tool, dict) else tool.plugin_id

    if tool_plugin_id != req.plugin_id:
        return ResponseModel(
            code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            message="plugin id is not match",
        )

    # tool is already a dict from database, not a Pydantic model
    tool_dict = tool if isinstance(tool, dict) else tool.model_dump()

    # Convert input_parameters to request_params with runtime and value
    if 'input_parameters' in tool_dict and tool_dict['input_parameters']:
        request_params = _input_parameters_to_request_params(tool_dict['input_parameters'])
        tool_dict['request_params'] = [param.model_dump() for param in request_params]

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="get plugin code success",
        data=PluginCodeInfoResponse(
            code_info=[PluginCodeInfo(**tool_dict)],
            total=1,
        )
    )


@with_exception_handling
def plugin_list_code(
        req: PluginListTool,
        current_user: dict
) -> ResponseModel:
    """获取插件code工具列表"""
    _ = check_user_space(req.space_id, current_user)

    logger.info("list plugin code")
    list_res, tool_list = plugin_repository.plugin_get(req.model_dump())
    list_result = ResponseModel(**list_res)
    if list_result.code != status.HTTP_200_OK and list_result.code != status.HTTP_404_NOT_FOUND:
        return ResponseModel(
            code=list_result.code,
            message=list_result.message,
        )

    code_infos: List[PluginCodeInfo] = []
    for info_dict in tool_list:
        info = PluginCodeInfo(**info_dict)
        if info.plugin_id == req.plugin_id:
            code_infos.append(info)

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="list plugin code success",
        data=PluginCodeInfoResponse(
            code_info=code_infos,
            total=len(code_infos),
        )
    )


def _compare_versions(ver1, ver2):
    v1 = version.parse(ver1)
    v2 = version.parse(ver2)

    if v1 < v2:
        return True
    return False


@with_exception_handling
def plugin_publish(
        req: PluginPublish,
        current_user: dict
) -> ResponseModel:
    """
    发布插件

    Args:
        req: 插件发布请求
        current_user: 当前用户信息

    Returns:
        ResponseModel: 发布结果
    """
    # 1. 校验Space_id是否有权限
    _ = check_user_space(req.space_id, current_user)

    plugin_query = {
        "plugin_id": req.plugin_id,
        "space_id": req.space_id
    }

    # 2. 获取最新版本信息
    res = plugin_repository.plugin_publish_get(plugin_query)
    get_version_result = ResponseModel(**res)
    logger.info(f"get version plugin info: {get_version_result}")

    if get_version_result.code == status.HTTP_404_NOT_FOUND:
        latest_version = "0.0.0"  # 初始版本
    elif get_version_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=get_version_result.code,
            message=f"Get versioned plugin with id {req.plugin_id} failed, error: {get_version_result.message}",
            data=None
        )
    else:
        latest_version_data = PluginPublishDBPd(**get_version_result.data)
        latest_version = latest_version_data.plugin_version

    # 3. 检查版本格式和递增性（除非强制发布）
    if not req.force:
        is_valid = _compare_versions(latest_version, req.plugin_version)
        if not is_valid:
            return ResponseModel(
                code=status.HTTP_400_BAD_REQUEST,
                message=f"Version check failed",
                data=None
            )

    # 4. 获取插件草稿内容
    plugin_draft_query = {
        "plugin_id": req.plugin_id,
        "space_id": req.space_id,
    }

    draft_result, tool_list = plugin_repository.plugin_get(plugin_draft_query)
    draft_response = ResponseModel(**draft_result)

    if draft_response.code != status.HTTP_200_OK:
        return ResponseModel(
            code=draft_response.code,
            message=f"Get plugin draft failed: {draft_response.message}",
            data=None
        )

    plugin_data = draft_response.data
    plugin_info = PluginBaseDBPd(**(plugin_data.model_dump()))

    # 5. 创建发布版本
    publish_data = {
        "plugin_id": req.plugin_id,
        "name": plugin_info.name,
        "desc": plugin_info.desc,
        "desc_mk": plugin_info.desc_mk,
        "plugin_version": req.plugin_version,
        "version_desc": req.version_desc,
        "url": plugin_info.url,
        "icon_uri": plugin_info.icon_uri,
        "plugin_type": plugin_info.plugin_type,
        "space_id": req.space_id,
        "inputs": plugin_info.inputs,
        "tools": tool_list,
        "create_time": milliseconds(),
        "update_time": milliseconds()
    }

    # 6. 保存发布版本
    publish_result = plugin_repository.plugin_publish_create(publish_data)
    publish_response = ResponseModel(**publish_result)

    if publish_response.code != status.HTTP_200_OK:
        return ResponseModel(
            code=publish_response.code,
            message=f"Create plugin publish version failed: {publish_response.message}",
            data=None
        )

    # 7. 返回发布结果
    return ResponseModel(
        code=status.HTTP_200_OK,
        message="Plugin published successfully",
        data=PluginPublishResponse(
            plugin_id=req.plugin_id,
            version=req.plugin_version,
            published_at=str(milliseconds())
        )
    )


@with_exception_handling
def plugin_publish_list(
        req: PluginId,
        current_user: dict
) -> ResponseModel:
    """
    发布插件

    Args:
        req: 插件发布列表请求
        current_user: 当前用户信息

    Returns:
        ResponseModel: 发布列表查询结果
    """
    # 1. 校验Space_id是否有权限
    _ = check_user_space(req.space_id, current_user)

    res = plugin_repository.plugin_publish_list(req.model_dump())
    list_result = ResponseModel(**res)
    logger.info(f"get plugin list from db result: {list_result}")
    if list_result.code != status.HTTP_200_OK and list_result.code != status.HTTP_404_NOT_FOUND:
        return ResponseModel(
            code=list_result.code,
            message=list_result.message,
        )

    infos: List[PluginPublishInfo] = []
    if list_result.data is not None:
        for info_dict in list_result.data:
            # 使用 from_db_with_mapping 将 inputs 映射为 request_params
            info = PluginPublishInfo.from_db_with_mapping(info_dict)
            infos.append(info)
    return ResponseModel(
        code=status.HTTP_200_OK,
        message="plugin publish list success",
        data=PluginPublishListResponse(plugin_infos=infos)
    )


@with_exception_handling
def plugin_publish_get(
        req: PluginId,
        current_user: dict
) -> ResponseModel:
    """获取发布插件信息"""
    _ = check_user_space(req.space_id, current_user)

    res = plugin_repository.plugin_publish_get(req.model_dump())
    canvas_result = ResponseModel(**res)
    logger.info(f"get publish plugin info from db result: {canvas_result}")
    if canvas_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=canvas_result.code,
            message=canvas_result.message,
        )
    # 使用 from_db_with_mapping 将 inputs 映射为 request_params
    plugin_info = PluginPublishInfo.from_db_with_mapping(canvas_result.data)
    return ResponseModel(
        code=status.HTTP_200_OK,
        message="get publish plugin success",
        data=PluginPublishInfoResponse(plugin_info=plugin_info)
    )


@with_exception_handling
def plugin_publish_delete(
        req: PluginId,
        current_user: dict
) -> ResponseModel:
    """删除已发布的插件"""
    _ = check_user_space(req.space_id, current_user)

    logger.info(f"delete publish plugin: {req}")
    res = plugin_repository.plugin_publish_delete(req.model_dump())
    delete_result = ResponseModel(**res)
    logger.info(f"delete publish plugin info in db result: {delete_result}")
    if delete_result.code != status.HTTP_200_OK:
        return ResponseModel(
            code=delete_result.code,
            message=delete_result.message,
        )

    return ResponseModel(
        code=status.HTTP_200_OK,
        message="delete publish plugin success",
    )


@with_exception_handling
def plugin_read_market_json(
        req: PluginList,
        current_user: dict
) -> ResponseModel:
    """
    读取backend目录下的config.json文件内容并以JSON字符串形式返回

    Args:
        req: 用户空间
        current_user: 当前用户信息

    Returns:
        ResponseModel: 包含JSON文件内容的响应模型
    """
    # 1. 校验Space_id是否有权限
    _ = check_user_space(req.space_id, current_user)

    # 2. 构造config.json文件路径
    base_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "../../conf/"))
    config_file_path = os.path.join(base_dir, "config.json")
    try:
        with open(config_file_path, 'r', encoding='utf-8') as f:
            json_content = json.load(f)
        json_content["VITE_PLUGIN_SERVICE_URL"] = os.getenv("VITE_PLUGIN_SERVICE_URL", "")
        json_string = json.dumps(json_content, ensure_ascii=False, indent=2)

        return ResponseModel(
            code=status.HTTP_200_OK,
            message="JSON file read successfully",
            data=json_string
        )
    except FileNotFoundError:
        logger.error(f"JSON file not found: {config_file_path}")
        return ResponseModel(
            code=status.HTTP_404_NOT_FOUND,
            message=f"JSON file not found: {config_file_path}",
            data=""
        )
    except json.JSONDecodeError as e:
        logger.error(f"Invalid JSON format in file: {config_file_path}, error: {str(e)}")
        return ResponseModel(
            code=status.HTTP_400_BAD_REQUEST,
            message=f"Invalid JSON format in file: {config_file_path}",
            data=""
        )
    except Exception as e:
        logger.error(f"Error reading JSON file: {str(e)}")
        return ResponseModel(
            code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            message=f"Error reading JSON file: {str(e)}",
            data=""
        )


@with_exception_handling
def plugin_tool_update_available(
        tool_id: str,
        space_id: str,
        available: bool,
        plugin_version: str = None
) -> ResponseModel:
    """
    更新工具的可用状态（供运行面调用）

    当工具执行成功时，调用此方法将工具的 available 字段设置为 True
    当工具执行失败时，调用此方法将工具的 available 字段设置为 False

    Args:
        tool_id: 工具ID
        space_id: 空间ID
        available: 工具是否可用（True=可用，False=不可用）
        plugin_version: 插件版本（可选，默认使用 __version_none__）

    Returns:
        ResponseModel: 更新结果

    Example:
        # 工具执行成功后调用
        plugin_tool_update_available("tool_123", "space_456", True)

        # 工具执行失败后调用
        plugin_tool_update_available("tool_123", "space_456", False)
    """
    result = tool_repository.tool_update_available(
        tool_id=tool_id,
        space_id=space_id,
        available=available,
        plugin_version=plugin_version
    )

    if result.get("code") == status.HTTP_200_OK:
        logger.info(
            f"Tool available status updated: tool_id={tool_id}, "
            f"space_id={space_id}, available={available}"
        )
        return ResponseModel(
            code=status.HTTP_200_OK,
            message="Tool available status updated successfully",
            data=result
        )
    else:
        logger.error(
            f"Failed to update tool available status: tool_id={tool_id}, "
            f"space_id={space_id}, error={result.get('message')}"
        )
        return ResponseModel(
            code=result.get("code", status.HTTP_500_INTERNAL_SERVER_ERROR),
            message=result.get("message", "Failed to update tool available status"),
            data=result
        )



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.

from abc import ABC, abstractmethod
from typing import Any, Dict, Optional, List, Union

from openjiuwen.core.common.logging import logger
from openjiuwen.core.workflow import Start, End, WorkflowComponent, SubWorkflowComponent
from openjiuwen.core.workflow import LoopComponent, LoopGroup, LoopBreakComponent, LoopSetVariableComponent
# from openjiuwen.core.workflow import ToolComponentConfig, ToolComponent # 下一个版本再用
from openjiuwen.core.workflow.workflow import Workflow as InvokableWorkflow
from openjiuwen.core.workflow import WorkflowCard
from openjiuwen.core.workflow import ComponentAbility

from openjiuwen_studio.core.common.dsl import PluginCodeConfig as DlPluginCodeConfig
from openjiuwen_studio.core.common.dsl import PluginType
from openjiuwen_studio.core.common.dsl import RestfulApiSchema as DlRestfulApiSchema
from openjiuwen_studio.core.common.dsl import ToolCompConfig, LoopConfig, ExecSubWfConfig, SetVariableConfig, \
    BaseFlow
from openjiuwen_studio.core.common.dsl import Workflow as DlWorkflow, Component, EndConfig, ComponentType
from openjiuwen_studio.core.executor.component.compile.intent_detection_comp_compiler import IntentDetectionCompCompiler
from openjiuwen_studio.core.executor.component.compile.llm_comp_compiler import LLMCompCompiler
from openjiuwen_studio.core.executor.component.compile.questioner_comp_compiler import QuestionerCompCompiler
from openjiuwen_studio.core.executor.component.compile.branch_comp_compiler import BranchCompCompiler
from openjiuwen_studio.core.executor.component.compile.code_comp_compiler import CodeCompCompiler
from openjiuwen_studio.core.executor.component.component_impl.empty_comp import EmptyComponent
from openjiuwen_studio.core.executor.component.compile.text_editor_comp_compiler import TextEditorCompCompiler
from openjiuwen_studio.core.executor.component.compile.user_input_comp_compiler import UserInputCompCompiler
from openjiuwen_studio.core.executor.component.compile.user_output_comp_compiler import UserOutputCompCompiler, \
    find_llm_to_stream_out, change_stream_input
from openjiuwen_studio.core.executor.component.compile.variable_merge_comp_compiler import VariableMergeCompCompiler
from openjiuwen_studio.core.executor.plugin.plugin_tools import ServiceTool, CodeTool
from openjiuwen_studio.core.executor.workflow.context import Context
from openjiuwen_studio.core.executor.workflow.pregel_graph_adapter import PregelGraphAdapter
from openjiuwen.core.graph.executable import Executable
from openjiuwen_studio.core.executor.component.component_impl.user_output_comp import UserOutputComponent
from openjiuwen_studio.core.executor.component.component_impl.tool_comp import ToolComponentConfig, ToolComponent


class IWorkflowLoader(ABC):
    @abstractmethod
    async def get_compiled_workflow(self, context: Context, id: str, version: str, space_id,
                                    current_user) -> InvokableWorkflow:
        pass


def set_comp_with_stream(
        self: InvokableWorkflow,
        comp_id: str,
        component: Union[Executable, WorkflowComponent],
        inputs_schema: dict = None,
        outputs_schema: dict = None,
        stream_inputs_schema: dict = None,
        stream_outputs_schema: dict = None,
        response_mode: str = None,
        wait_for_all: bool = None
) -> InvokableWorkflow:
    """Workflow添加支持流式处理的组件添加方法"""

    # 实现逻辑与方式1相同
    if wait_for_all is None:
        wait_for_all = False
    comp_ability = []
    if response_mode is not None and "streaming" == response_mode:
        if inputs_schema:
            comp_ability = [ComponentAbility.STREAM]
        self._is_streaming = True
        if stream_inputs_schema:
            comp_ability.append(ComponentAbility.TRANSFORM)
            if isinstance(component, End):
                component.set_mix()
            wait_for_all = True
    else:
        if inputs_schema:
            comp_ability = [ComponentAbility.INVOKE]
        if stream_inputs_schema:
            comp_ability.append(ComponentAbility.COLLECT)
            if isinstance(component, UserOutputComponent):
                component.set_mix()
            wait_for_all = True

    self.add_workflow_comp(
        comp_id,
        component,
        wait_for_all=wait_for_all,
        inputs_schema=inputs_schema,
        comp_ability=comp_ability,
        outputs_schema=outputs_schema,
        stream_inputs_schema=stream_inputs_schema,
        stream_outputs_schema=stream_outputs_schema,
    )
    logger.debug(f"comp set comp_ability: {comp_ability}")
    return self


InvokableWorkflow.set_comp_with_stream = set_comp_with_stream


class Workflow:
    # 组件编译器映射表
    COMPILER_HANDLERS = {
        ComponentType.COMPONENT_TYPE_LLM: '_compile_llm_component',
        ComponentType.COMPONENT_TYPE_QUESTION: '_compile_question_component',
        ComponentType.COMPONENT_TYPE_INTENT: '_compile_intent_component',
        ComponentType.COMPONENT_TYPE_INPUT: '_compile_input_component',
        ComponentType.COMPONENT_TYPE_OUTPUT: '_compile_output_component',
        ComponentType.COMPONENT_TYPE_TEXT_EDITOR: '_compile_text_editor_component',
        ComponentType.COMPONENT_TYPE_VARIABLE_MERGE: '_compile_variable_merge_component',
        ComponentType.COMPONENT_TYPE_CODE: '_compile_code_component',
    }

    # 特殊组件类型（需要异步处理或特殊参数）
    SPECIAL_COMPONENT_TYPES = {
        ComponentType.COMPONENT_TYPE_IF,
        ComponentType.COMPONENT_TYPE_SUB_WORKFLOW,
        ComponentType.COMPONENT_TYPE_LOOP,
        ComponentType.COMPONENT_TYPE_PLUGIN,
    }

    # 空组件类型
    EMPTY_COMPONENT_TYPES = {
        ComponentType.COMPONENT_TYPE_EMPTY,
        ComponentType.COMPONENT_TYPE_CONTINUE,
        ComponentType.COMPONENT_TYPE_EMPTY_START,
        ComponentType.COMPONENT_TYPE_EMPTY_END,
    }

    def __init__(
            self,
            dl_workflow: DlWorkflow,
            space_id: str,
            current_user: Dict[str, Any]
    ) -> None:
        logger.info(f"first dsl: {dl_workflow.model_dump_json()}")
        for component in dl_workflow.components:
            if component.type == ComponentType.COMPONENT_TYPE_LOOP:
                loop_config = LoopConfig.model_validate(component.configs)
                loop_graph_adapter = PregelGraphAdapter(loop_config.loop_body)
                loop_config.loop_body = loop_graph_adapter.convert()
                component.configs = loop_config.model_dump()
        self.id = dl_workflow.id
        self.version = dl_workflow.version
        self.name = dl_workflow.name
        self.inputs = dl_workflow.inputs
        graph_adapter = PregelGraphAdapter(dl_workflow)
        self.dl_workflow = graph_adapter.convert()
        logger.info(f"second dsl: {self.dl_workflow.model_dump_json()}")
        self.space_id = space_id
        self.current_user = current_user
        self.need_stream_output_comp = {}

    async def process_components(
            self,
            context: Context,
            flow: Any,
            workflow_dl: BaseFlow,
            loader: Optional[IWorkflowLoader] = None
    ) -> None:
        dl_wf_components = workflow_dl.components
        for comp in dl_wf_components:
            if comp.id in self.dl_workflow.start_id:
                compiled_comp = Start()
                flow.set_start_comp(comp.id, compiled_comp, comp.inputs)
            elif comp.id in self.dl_workflow.end_id:
                stream_inputs_schema = None
                batch_inputs_schema = None
                response_mode = None
                if comp.configs:
                    end_config = EndConfig.model_validate(comp.configs)
                    if end_config.response_template:
                        compiled_comp = End({"response_template": end_config.response_template})
                    else:
                        compiled_comp = End()
                    # 打开流式输出开关
                    if end_config.stream_output:
                        find_llm_to_stream_out(comp.id, comp.inputs, self.need_stream_output_comp)
                        stream_inputs_schema, batch_inputs_schema = change_stream_input(comp.inputs)
                        if stream_inputs_schema:
                            response_mode = "streaming"
                    else:
                        # 非流式输出模式下，直接使用原始输入
                        batch_inputs_schema = comp.inputs

                else:
                    compiled_comp = End()
                    # 没有配置时，直接使用原始输入
                    batch_inputs_schema = comp.inputs
                logger.debug(f"set_end_comp inputs_schema: {batch_inputs_schema}, "
                             f"stream_inputs_schema: {stream_inputs_schema}, "
                             f"response_mode: {response_mode}")
                flow.set_end_comp(comp.id, compiled_comp, inputs_schema=batch_inputs_schema,
                                  stream_inputs_schema=stream_inputs_schema, response_mode=response_mode)
            else:
                compiled_comp = await self.compile_component(context, workflow_dl, comp, loader)
                if compiled_comp is None:
                    continue
                if comp.type == ComponentType.COMPONENT_TYPE_OUTPUT:
                    if any(
                            (isinstance(value, str) and comp.id == value) or
                            (isinstance(value, list) and comp.id in value)
                            for value in self.need_stream_output_comp.values()
                    ):
                        stream_inputs, new_inputs = change_stream_input(comp.inputs)
                        if stream_inputs:
                            response_mode = "streaming"
                        else:
                            response_mode = None
                        logger.debug(f"set_comp_with_stream stream_inputs: {stream_inputs}, new_inputs: {new_inputs}")
                        flow.set_comp_with_stream(comp.id, compiled_comp, inputs_schema=new_inputs,
                                                  stream_inputs_schema=stream_inputs, response_mode=response_mode)
                        continue
                flow.add_workflow_comp(comp.id, compiled_comp, inputs_schema=comp.inputs)
        return flow

    async def compile(
            self,
            context: Context,
            loader: Optional[IWorkflowLoader] = None
    ) -> InvokableWorkflow:
        card = WorkflowCard(
            id=self.id,
            version=self.version,
            name=self.name,
            input_params=self.inputs
        )
        flow = InvokableWorkflow(card=card)

        flow = await self.process_components(context, flow, self.dl_workflow, loader)
        flow = await self.process_stream_connections(flow)
        flow = await self.process_connections(flow, self.dl_workflow.connections)
        return flow

    async def compile_component(
            self,
            context: Context,
            workflow_dl: BaseFlow,
            comp: Component,
            loader: Optional[IWorkflowLoader] = None
    ) -> Any:
        """使用简单注册机制编译组件"""

        # 1. 处理空组件
        if comp.type in self.EMPTY_COMPONENT_TYPES:
            return EmptyComponent()

        # 2. 处理BREAK组件
        if comp.type == ComponentType.COMPONENT_TYPE_BREAK:
            return LoopBreakComponent()

        # 3. 处理特殊组件（保持原有逻辑）
        if comp.type in self.SPECIAL_COMPONENT_TYPES:
            return await self._compile_special_component(context, workflow_dl, comp, loader)

        # 4. 使用注册表编译标准组件
        handler_name = self.COMPILER_HANDLERS.get(comp.type)
        if handler_name:
            handler = getattr(self, handler_name)
            return await handler(comp, workflow_dl)
        else:
            logger.warning(f"Unsupported component type: {comp.type}")
            return None

    # 各个组件的编译方法
    async def _compile_llm_component(self, comp: Component, workflow_dl: BaseFlow):
        """编译LLM组件"""
        llm_compiler = LLMCompCompiler(comp.configs)
        return llm_compiler.compile()

    async def _compile_question_component(self, comp: Component, workflow_dl: BaseFlow):
        """编译提问者组件"""
        questioner_compiler = QuestionerCompCompiler(comp.configs)
        return questioner_compiler.compile()

    async def _compile_intent_component(self, comp: Component, workflow_dl: BaseFlow):
        """编译意图检测组件"""
        compiler = IntentDetectionCompCompiler(comp, workflow_dl.connections)
        return compiler.compile()

    async def _compile_input_component(self, comp: Component, workflow_dl: BaseFlow):
        """编译用户输入组件"""
        userinput_compiler = UserInputCompCompiler(comp.configs, comp.id)
        return userinput_compiler.compile()

    async def _compile_output_component(self, comp: Component, workflow_dl: BaseFlow):
        """编译用户输出组件"""
        useroutput_compiler = UserOutputCompCompiler(comp.id, comp.configs, comp.inputs, self.need_stream_output_comp)
        output_component, self.need_stream_output_comp = useroutput_compiler.compile()
        return output_component

    async def _compile_text_editor_component(self, comp: Component, workflow_dl: BaseFlow):
        """编译文本编辑器组件"""
        texteditor_compiler = TextEditorCompCompiler(comp.id, comp.configs, comp.outputs)
        return texteditor_compiler.compile()

    async def _compile_variable_merge_component(self, comp: Component, workflow_dl: BaseFlow):
        """编译变量合并组件"""
        varimerge_compiler = VariableMergeCompCompiler(comp.configs, comp.id)
        return varimerge_compiler.compile()

    async def _compile_code_component(self, comp: Component, workflow_dl: BaseFlow):
        """编译代码组件"""
        code_compiler = CodeCompCompiler(comp.id, comp.configs, workflow_dl.connections)
        return code_compiler.compile()

    async def _compile_special_component(self, context: Context, workflow_dl: BaseFlow, comp: Component, loader):
        """处理特殊组件（保持原有逻辑）"""
        if comp.type == ComponentType.COMPONENT_TYPE_IF:
            branch_compiler = BranchCompCompiler(comp.id, comp.branches, workflow_dl.connections)
            return branch_compiler.compile()

        elif comp.type == ComponentType.COMPONENT_TYPE_SUB_WORKFLOW:
            return await self._create_exec_sub_workflow_component(context, comp.configs, loader)

        elif comp.type == ComponentType.COMPONENT_TYPE_LOOP:
            return await self._create_loop_component(context, comp.configs, comp.outputs, loader)

        elif comp.type == ComponentType.COMPONENT_TYPE_PLUGIN:
            return await self._compile_plugin_component(comp)

        return None

    async def _compile_plugin_component(self, comp: Component):
        """编译插件组件"""
        tool_config = ToolCompConfig.model_validate(comp.configs)
        if tool_config.type == PluginType.SERVICE:
            plugin_tool = ServiceTool(DlRestfulApiSchema.model_validate(tool_config.tool)).compile()
        else:
            plugin_tool = CodeTool(DlPluginCodeConfig.model_validate(tool_config.tool)).compile()
        tool_config = ToolComponentConfig(tool_id=comp.id)
        # 不使用 Runner.resource_mgr.add_tool 因为不能重复添加
        return ToolComponent(tool_config).bind_tool(plugin_tool)

    async def process_stream_connections(self,
                                         flow: Any) -> Any:
        if not self.need_stream_output_comp:
            return flow
        for source_id, target_id in self.need_stream_output_comp.items():

            if isinstance(target_id, list):
                for tid in target_id:
                    logger.debug(f"add_stream_connection source_id: {source_id}, target_id: {tid}")
                    flow.add_stream_connection(source_id, tid)
            else:
                logger.debug(f"add_stream_connection source_id: {source_id}, target_id: {target_id}")
                flow.add_stream_connection(source_id, target_id)

        return flow

    async def do_add_connection(self, flow, source, target):
        skip_connection = False
        for source_id, target_id in self.need_stream_output_comp.items():
            if (source == source_id) and (target == target_id or target in target_id):
                skip_connection = True
                break
        if not skip_connection:
            logger.debug(f"add_connection source: {source}, target: {target}")
            flow.add_connection(source, target)
        return flow

    async def process_connections(
            self,
            flow: Any,
            dl_wf_connections: List[Any]
    ) -> Any:
        for conn in dl_wf_connections:
            # 跳过分支连接，因为分支连接由组件内部的add_branch方法处理
            # 只有非分支连接需要被添加到flow中
            if conn.branch_id:
                continue
            # 检查 conn.source 是否包含 self.need_stream_output_comp 中的任何 key
            source_list = conn.source if isinstance(conn.source, list) else [conn.source]
            need_stream_sources = set(self.need_stream_output_comp.keys())
            has_stream_source = bool(set(source_list) & need_stream_sources)

            if has_stream_source:
                # 如果 source 在 need_stream_output_comp 中，需要遍历出没有加过add_stream_connection的连接
                if isinstance(conn.source, list):
                    for sid in conn.source:
                        flow = await self.do_add_connection(flow, sid, conn.target)
                else:
                    flow = await self.do_add_connection(flow, conn.source, conn.target)
            else:
                # 如果 source 不在 need_stream_output_comp 中，直接添加连接
                logger.debug(f"add_connection source: {conn.source}, target: {conn.target}")
                flow.add_connection(conn.source, conn.target)
        return flow

    async def _create_exec_sub_workflow_component(
            self,
            context: Context,
            configs: Any,
            loader: Optional[IWorkflowLoader]
    ) -> Any:
        sub_wf_info = ExecSubWfConfig.model_validate(configs).sub_workflow_info
        sub_id = sub_wf_info.id
        sub_version = sub_wf_info.version
        sub_workflow = await loader.get_compiled_workflow(Context(context),
                                                          sub_id, sub_version, self.space_id, self.current_user)
        return SubWorkflowComponent(sub_workflow)

    async def _create_loop_component(
            self,
            context: Context,
            comp_configs: Any,
            comp_outputs: Any,
            loader: Optional[IWorkflowLoader] = None

    ) -> Any:
        # create loop component
        loop_group = LoopGroup()
        loop_config = LoopConfig.model_validate(comp_configs)
        loop_body = loop_config.loop_body
        loop_group.start_nodes(loop_body.start_id)
        loop_group.end_nodes(loop_body.end_id)
        loop_group = await self.process_components(context, loop_group, loop_body, loader)
        loop_group = await self.process_connections(loop_group, loop_body.connections)

        # 添加SetVariable组件
        for comp in loop_body.components:
            if comp.type == ComponentType.COMPONENT_TYPE_SET_VARIABLE:
                set_variable_config = SetVariableConfig.model_validate(comp.configs)
                set_variable_comp = LoopSetVariableComponent(set_variable_config.inter_variable)
                loop_group.add_workflow_comp(comp.id, set_variable_comp)

        return LoopComponent(loop_group, comp_outputs)



#!/usr/bin/python3.10
# coding: utf-8
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved
from typing import Any, List

from openjiuwen.core.workflow import WorkflowComponent, Input, Output
from openjiuwen.core.context_engine import ModelContext
from openjiuwen.core.workflow.components import Session
from openjiuwen.core.common.logging import logger

from openjiuwen_studio.core.common.dsl import UserInputsConfig, UserInputElem
from openjiuwen_studio.core.common.exceptions import JiuWenExecuteException
from openjiuwen_studio.core.common.status_code import StatusCode


# {
#     "id": "UserInput1",
#     "version": "",
#     "name": "",
#     "description": "",
#     "type": "jiuwen.UserInputComponent",
#     "typeVersion": "1.0.0",
#     "inputs": {},
#     "outputs": {},
#     "configs": {
#        "inputs": {
#         {"input_name": "AA", "description": "", "type":"str", "required": True},
#         {"input_name": "BB", "description": "", "type":"int", "required": True},
#         {"input_name": "CC", "description": "", "type":"bool", "required": False}
#        }
#     }
# }

# user_input = InteractiveInput()
# user_input.update(self.node_id, {"AA": "ishshe", "BB": 444, "CC": 1})


class UserInputComponent(WorkflowComponent):
    def __init__(self, conf: UserInputsConfig) -> None:
        super().__init__()
        self.input_conf_list: List[UserInputElem] = conf.inputs

    async def invoke(self, inputs: Input, session: Session, context: ModelContext) -> Output:
        result = await session.interact(self.input_conf_list)
        for input_elem in self.input_conf_list:
            if not isinstance(input_elem, UserInputElem):
                raise ValueError("Node data type is wrong")

            value = result.get(input_elem.input_name)
            if value is None:
                if input_elem.required is True:
                    raise JiuWenExecuteException(
                        StatusCode.USERINPUT_COMPONENT_INVOKE_ERROR.code,
                        StatusCode.USERINPUT_COMPONENT_INVOKE_ERROR.errmsg,
                    )
                continue

            if input_elem.type:
                try:
                    if input_elem.type == "integer":
                        result[input_elem.input_name] = int(value)
                    elif input_elem.type == "number":
                        result[input_elem.input_name] = float(value)
                    elif input_elem.type == "string":
                        result[input_elem.input_name] = str(value)
                    elif input_elem.type == "boolean":
                        if isinstance(value, str):
                            if value.lower() == "true":
                                result[input_elem.input_name] = True
                            elif value.lower() == "false":
                                result[input_elem.input_name] = False
                        else:
                            result[input_elem.input_name] = bool(value)
                except Exception as e:
                    logger.warning(
                        f"Failed to convert input value '{value}' to type '{input_elem.type}' "
                        f"for field '{input_elem.input_name}': {e}"
                    )
        return result



import secrets
import string

from threading import Lock
from openjiuwen_studio.core.manager.redis_manager.redis_client import redis_manager_str as redis_manager
from fastapi import HTTPException
from cachetools import TTLCache


class TTLCacheRateLimiter:
    LOGIN_NAME = "login"
    REGISTER_NAME = "register"
    LOGIN_TIME_RANGE = 60
    REGISTER_TIME_RANGE = 3600
    MAX_SIZE = 10000

    def __init__(self):
        # 创建两个缓存：登录/注册
        self.caches = {
            TTLCacheRateLimiter.LOGIN_NAME:
                TTLCache(maxsize=TTLCacheRateLimiter.MAX_SIZE, ttl=TTLCacheRateLimiter.LOGIN_TIME_RANGE),
            TTLCacheRateLimiter.REGISTER_NAME:
                TTLCache(maxsize=TTLCacheRateLimiter.MAX_SIZE, ttl=TTLCacheRateLimiter.REGISTER_TIME_RANGE)
        }
        self.locks = {name: Lock() for name in self.caches}

    def allow_request(self, cache_name: str, key: str, max_requests: int) -> bool:
        cache = self.caches[cache_name]
        lock = self.locks[cache_name]
        with lock:
            # 获取当前计数，若 key 不存在则默认为0
            current = cache.get(key, 0)
            if current < max_requests:
                cache[key] = current + 1
                return True
            else:
                return False


class SecurityManager:
    """安全管理器：处理验证码、登录失败锁定"""
    _REG_CODE_KEY = "auth:reg:code:{email}"
    _REG_LIMIT_KEY = "auth:reg:limit:{email}"
    _RESET_CODE_KEY = "auth:reset:code:{email}"
    _RESET_LIMIT_KEY = "auth:reset:limit:{email}"
    _FAIL_COUNT_KEY = "auth:fail:count:{email}"
    _VERIFY_ATTEMPT_KEY = "auth:verify:attempt:{email}:{action_type}"
    _LOGIN_RATE_LIMIT_KEY = "rate_limit:login:{client_ip}"
    _REGISTER_RATE_LIMIT_KEY = "rate_limit:register:{client_ip}"
    MAX_VERIFY_ATTEMPTS = 5

    MAX_LOGIN_ATTEMPTS = 5
    LOCK_TIME = 1800  # 30分钟锁定
    CODE_EXPIRE = 600 # 10分钟验证码有效
    LIMIT_EXPIRE = 60 # 60秒发送频率限制
    LIMITER = TTLCacheRateLimiter()

    # 邮箱验证码相关操作
    @classmethod
    def generate_and_save_code(cls, email: str, action_type: str = "reg") -> str:
        """生成验证码并存入 Redis"""
        code = ''.join(secrets.choice(string.digits) for _ in range(6))
        template = cls._REG_CODE_KEY if action_type == "reg" else cls._RESET_CODE_KEY
        limit_template = cls._REG_LIMIT_KEY if action_type == "reg" else cls._RESET_LIMIT_KEY
        # 存储验证码和频率限制
        redis_manager.set_ex(template.format(email=email), code, cls.CODE_EXPIRE)
        redis_manager.set_ex(limit_template.format(email=email), "1", cls.LIMIT_EXPIRE)
        return code

    @classmethod
    def verify_code(cls, email: str, input_code: str, action_type: str = "reg") -> bool:
        """校验验证码，通过后立即销毁"""
        template = cls._REG_CODE_KEY if action_type == "reg" else cls._RESET_CODE_KEY
        attempt_key = cls._VERIFY_ATTEMPT_KEY.format(email=email, action_type=action_type)
        attempt_count = redis_manager.get(attempt_key)
        key = template.format(email=email)
        if attempt_count and int(attempt_count) >= cls.MAX_VERIFY_ATTEMPTS:
            redis_manager.delete(key) # 超过重试上限后销毁验证码
            redis_manager.delete(attempt_key) # 清除重试次数
            return False
        saved_code = redis_manager.get(key)
        
        if saved_code and saved_code == input_code:
            redis_manager.delete(key) # 验证通过立即销毁
            redis_manager.delete(attempt_key) # 清除重试次数
            return True
        # 失败次数累计
        redis_manager.incr(attempt_key, cls.CODE_EXPIRE)
        return False

    @classmethod
    def rate_limit(cls, email: str, action_type: str = "reg") -> bool:
        """检查邮件发送是否处于受限状态（频率过快）"""
        template = cls._REG_LIMIT_KEY if action_type == "reg" else cls._RESET_LIMIT_KEY
        return redis_manager.exists(template.format(email=email))

    # 锁定计数相关逻辑
    @classmethod
    def get_lock_info(cls, email: str) -> dict:
        """获取账号锁定状态信息"""
        count = redis_manager.get(cls._FAIL_COUNT_KEY.format(email=email))
        count = int(count) if count else 0
        return {
            "is_locked": count >= cls.MAX_LOGIN_ATTEMPTS,
            "remaining_attempts": max(0, cls.MAX_LOGIN_ATTEMPTS - count),
            "current_fail_count": count
        }

    @classmethod
    def record_login_failure(cls, email: str):
        """记录登录失败，第一次失败时设置过期时间，后续不重置时间"""
        key = cls._FAIL_COUNT_KEY.format(email=email)
        if not redis_manager.exists(key):
            # 第一次失败，设置过期时间
            redis_manager.incr(key, expire=cls.LOCK_TIME)
        else:
            # 后续失败只自增，不重置过期时间
            redis_manager.incr(key)

    @classmethod
    def clear_auth_status(cls, email: str):
        """清理安全限制状态(重置密码或登录成功后)"""
        redis_manager.delete(cls._FAIL_COUNT_KEY.format(email=email))

    @classmethod
    def login_rate_limit(cls, client_ip: str, max_requests: int = 10):
        """
        检查是否超过登录限流
        """
        if not cls.LIMITER.allow_request(TTLCacheRateLimiter.LOGIN_NAME,
                                        cls._LOGIN_RATE_LIMIT_KEY.format(client_ip=client_ip), max_requests):
            raise HTTPException(status_code=429, detail=f"登录过于频繁，请 {TTLCacheRateLimiter.LOGIN_TIME_RANGE} 秒后再试")
        
    @classmethod
    def register_rate_limit(cls, client_ip: str, max_requests: int = 5):
        """
        检查是否超过注册限流
        """
        if not cls.LIMITER.allow_request(TTLCacheRateLimiter.REGISTER_NAME,
                                         cls._REGISTER_RATE_LIMIT_KEY.format(client_ip=client_ip), max_requests):
            raise HTTPException(status_code=429, detail=f"注册过于频繁，请 {TTLCacheRateLimiter.REGISTER_TIME_RANGE} 秒后再试")



#!/usr/bin/env python3.10
# coding: utf-8
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.

from typing import Any, Dict

from openjiuwen_studio.core.common.dsl import VariMergeConfig
from openjiuwen_studio.core.executor.component.component_impl.variable_merge_comp import VariableMergeComponent
from openjiuwen_studio.core.executor.component.compile.base_comp_compiler import BaseCompCompiler
from openjiuwen_studio.core.common.exceptions import JiuWenExecuteException
from openjiuwen_studio.core.common.status_code import StatusCode


class VariableMergeCompCompiler(BaseCompCompiler):
    def __init__(self, comp_config_dict: Dict[str, Any], node_id: str) -> None:
        super().__init__()
        self.comp_config_dict: Dict[str, Any] = comp_config_dict
        self.node_id = node_id

    def compile(self) -> VariableMergeComponent:
        if not self.comp_config_dict:
            raise JiuWenExecuteException(
                StatusCode.VARIABLE_MERGE_COMP_COMPILER_ERROR.code,
                StatusCode.VARIABLE_MERGE_COMP_COMPILER_ERROR.errmsg.format(
                    msg=f"node [{self.node_id}] is empty, please check!"),
                node_id=self.node_id
            )
        merge_config = VariMergeConfig.model_validate(self.comp_config_dict)
        return VariableMergeComponent(merge_config)


#!/usr/bin/python3.10
# -*- coding: utf-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.

import functools
import inspect
import json
import logging

from starlette.responses import StreamingResponse, JSONResponse

logger = logging.getLogger(__name__)

# 业务异常类型，其消息可安全返回给客户端
SAFE_EXCEPTION_TYPES = (ValueError, PermissionError)


def _is_safe_exception(exc: Exception) -> bool:
    """判断异常消息是否可以安全返回给客户端（业务异常或已知安全类型）"""
    return hasattr(exc, "status_code") or isinstance(exc, SAFE_EXCEPTION_TYPES)


def _get_safe_message(exc: Exception) -> str:
    """获取可安全返回给客户端的错误消息，内部异常仅记录日志并返回通用提示"""
    if _is_safe_exception(exc):
        return str(exc)
    logger.exception("Unhandled internal exception: %s", exc)
    return "Internal server error"


def handle_exceptions(response_model=JSONResponse):
    """
    通用异常装饰器
    - 同步函数的response_model不能是StreamingResponse，推荐使用默认数据类型或不传
    - 若被装饰函数返回类型是 StreamingResponse → 用 SSE 错误帧
    - 否则用 response_model 或 JSONResponse
    """
    def decorator(func):
        is_stream = inspect.signature(func).return_annotation is StreamingResponse
        if inspect.iscoroutinefunction(func):
            @functools.wraps(func)
            async def async_wrapper(*args, **kwargs):
                try:
                    result = await func(*args, **kwargs)
                    # 如果是StreamingResponse，需要处理生成器内部异常
                    if isinstance(result, StreamingResponse):
                        # 创建一个包装器来捕获生成器异常
                        async def error_catching_generator(original_gen):
                            try:
                                async for chunk in original_gen:
                                    yield chunk
                            except Exception as e:
                                # 将异常转换为SSE错误帧，不向客户端泄露内部异常详情
                                code = getattr(e, "code", getattr(e, "status_code", 500))
                                msg = _get_safe_message(e)
                                error_data = {
                                    "error": True,
                                    "code": code,
                                    "message": msg
                                }
                                yield f"data: {json.dumps(error_data, ensure_ascii=False)}\n\n"

                        # 包装原始body_iterator
                        wrapped_gen = error_catching_generator(result.body_iterator)

                        # 返回新的StreamingResponse
                        return StreamingResponse(
                            wrapped_gen,
                            media_type=result.media_type,
                            headers=result.headers,
                            status_code=result.status_code
                        )

                    return result
                except Exception as e:
                    return _build_error(e, is_stream, response_model)
            return async_wrapper

        @functools.wraps(func)
        def sync_wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                return _build_error(e, is_stream, response_model)
        return sync_wrapper
    return decorator


def _build_error(exc, is_stream: bool, response_model):
    code = getattr(exc, "status_code", 500)
    msg = _get_safe_message(exc)

    if is_stream:
        # SSE 错误帧
        data = f"data: {json.dumps({'code': code, 'msg': msg})}\n\n"
        return StreamingResponse(iter([data]), media_type="text/event-stream")

    # 普通 JSON
    if response_model is JSONResponse:
        return JSONResponse(status_code=code, content={"code": code, "msg": msg})

    # 检查response_model是否有code和msg字段（用于错误响应）
    try:
        # 尝试创建带有code和msg字段的响应
        return response_model(code=code, msg=msg)
    except (TypeError, ValueError):
        # 如果失败，说明这不是一个错误响应模型，返回标准JSON错误
        return JSONResponse(status_code=code, content={"code": code, "msg": msg})



from typing import Optional, Any

from pydantic import Field, BaseModel, ConfigDict

from openjiuwen_studio.models.agent import AgentBaseDBPd
from openjiuwen_studio.schemas.agent import AGENT_NAME_MAX_SIZE


class AgentWorkflowInputNode(BaseModel):
    type: str
    description: str
    required: bool


class AgentWorkflowInputs(BaseModel):
    type: Optional[str] = Field("")
    properties: dict[str, AgentWorkflowInputNode] = Field(default_factory=dict)


class AgentWorkflowListNodeBase(BaseModel):
    workflow_id: str = Field(..., min_length=1, max_length=100, alias="id")
    workflow_version: str = Field(..., min_length=1, max_length=100, alias="version")
    workflow_name: str = Field(..., min_length=1, max_length=255, alias="name")
    description: str = Field(..., min_length=1, max_length=500, alias="desc")

    class Config:
        populate_by_name = True


class AgentWorkflowListNode(AgentWorkflowListNodeBase):
    pass


class AgentModelListNode(BaseModel):
    api_key: str = Field("")
    api_base: str = Field("")
    model_provider: str = Field("", min_length=1, max_length=100)
    model_id: int = Field(..., ge=0, le=999999, alias="id")
    model_name: str = Field(..., min_length=1, max_length=100, alias="name")
    model_type: str = Field(..., min_length=1, max_length=100, alias="type")
    temperature: float = Field(..., ge=0.0, le=2.0)
    top_p: float = Field(..., ge=0.0, le=1.0)
    streaming: bool = Field(False)
    max_tokens: int = Field(..., ge=1)
    timeout: int = Field(..., ge=1)

    class Config:
        populate_by_name = True


class AgentOptionInfo(BaseModel):
    workflow_list: list[AgentWorkflowListNode] = Field(default_factory=list)
    model_list: list[AgentModelListNode] = Field(default_factory=list)


class SingleAgentData(BaseModel):
    agent_info: AgentBaseDBPd = Field(description="agent基础信息")
    agent_option_info: AgentOptionInfo = Field(description="agent可选项信息")


class AgentItem(BaseModel):
    agent_id: str = Field(..., min_length=1, max_length=100, alias="id")
    agent_name: str = Field(..., min_length=1, max_length=AGENT_NAME_MAX_SIZE, alias="name")
    agent_version: str = Field(..., min_length=1, max_length=100, alias="version")
    agent_type: str = Field(..., min_length=1, max_length=30, alias="type")
    description: str = Field(..., min_length=1, max_length=500, alias="desc")
    icon: str = Field(..., min_length=1, max_length=100)
    status: str = Field(..., min_length=1, max_length=100)
    model_name: str = Field(..., min_length=1, max_length=100)
    last_activate: str = Field(..., min_length=1, max_length=100)
    usage_count: int = Field(..., description="用户使用的次数")
    tags: list[str] = Field(default_factory=list)
    create_time: int
    update_time: int
    api_endpoint: str = Field(..., min_length=1, max_length=100)

    class Config:
        populate_by_name = True


class AgentListPagination(BaseModel):
    page: int = Field(..., ge=0, le=1000)
    page_size: int = Field(..., ge=0, le=1000)
    total: int = Field(..., ge=0, le=1000)
    total_pages: int = Field(..., ge=0, le=1000)


class AgentListInfo(BaseModel):
    agent_items: list[AgentItem] = Field(default_factory=list, description="agent列表信息")
    pagination: AgentListPagination
